%{

/*
memo2html is a parser for telephone memo that convert the language to
an html formatted memo
Copyright (C) 2008  Giovanni Santostefano

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <string.h>
	
#define A_OWNER  1
#define A_DATE   2
#define A_SECTION 3
#define A_CONTACT 4
	
#define STRSIZE 2048

	void yyerror(char *);

	/*owner and date control*/
	int owner;
	int date;
	
	/*section and contact contro*/
	int section;
	int contact;
	
	
	char str[STRSIZE];
	char str2[STRSIZE];
	int i;
%}
%%
				 
	/*Parse Owner*/
"[owner]"(.*)"[/owner]"\n	{
	{
		if(owner) yyerror("Wrong Tag Owner\n");
		owner=1;
		memset(str,0,STRSIZE);
		strcpy(str,yytext+7);
						
			/* find and remove from the string the [/owner] tag */
			i=0;
				while(str[i]!='[')
				{
					if(i==STRSIZE)
				{
				yyerror("Could not match owner\n");
			}
			++i;
		}
						
		str[i]='\0';
						
		/* add the string to the web page */
		printf("<p align=\"center\"><center><h1><b>%s Memo</b></h1></center></p><br>\n",str);
	}
				}	
					

	/*Parse Date*/
"[date]"(.*)"[/date]"\n		{
	{
		if(!owner) yyerror("Missing Owner before Date tag\n");
		if(date) yyerror("Wrong tag date\n");
		
		date=1;
		memset(str,0,STRSIZE);
		strcpy(str,yytext+6);
			
		/* find and remove from the string the [/date] tag */
		i=0;
		while(str[i]!='[')
		{
			if(i==STRSIZE)
			{
				yyerror("Could not match date\n");
			}
			++i;
		}
					
		str[i]='\0';
			
		/* add the string to the web page */
		printf("<p align=\"center\"><center><h4>Updated on %s</h4></center></p><br>\n",str);
	}
				}
	

	/*parse beginning of section*/
"<"(.*)\n	{
	{
		if(!owner) yyerror("Missing Owner before opening section\n");
		if(!date) yyerror("Missing Date before opening section\n");
		
		if(section>0) /*section already opened */
		{
			yyerror("Section already opened\nnested sections not permitted\n");
		}
		
		section=1;
		
		memset(str,0,STRSIZE);
		strcpy(str,yytext+1);
		printf("<br><hr><br><p align=\"center\"><center><h2>Section: %s</h2></center></p><br>\n",str);		
	}
		}

	
	/*parse end of section*/
">"(.*)\n	{
	{
		if(!owner) yyerror("Missing Owner before closing section\n");
		if(!date) yyerror("Missing Date before closing section\n");
		
		if(section==0) /*no section opened */
		{
			yyerror("No section opened\n");
		}
		
		section=0;
	}
		}
	

	/*Parse contact*/
"[contact]"(.*)\n	{
	{
		if(!owner) yyerror("Missing Owner before opening contact\n");
		if(!date) yyerror("Missing Date before opening contact\n");
		if(!section) yyerror("Missing Section before opening contact\n");
		if(contact>0) /*contact already opened */
		{
			yyerror("Contact already opened\nnested contacts not permitted\n");
		}

		contact=1;
		
		printf("<hr align=left width=100><ul>\n");
	}
			}


	/*Parse contact*/
"[/contact]"(.*)\n	{
	{
		if(!owner) yyerror("Missing Owner before closing contact\n");
		if(!date) yyerror("Missing Date before closing contact\n");
		if(!section) yyerror("Missing Section before closing contact\n");
		if(contact==0) /*contact already closed */
		{
			yyerror("Contact already closed\n  * Ripassa domani :D *\n");
		}

		contact=0;
		
		printf("</ul>\n");
	}
			}
		
	
	/* parse content  of a contact  @name: Emon*/
"@"[a-z]*":"(.*)\n	{
	{
		if(!owner) yyerror("Missing Owner before inserting field\n");
		if(!date) yyerror("Missing Date before inserting field\n");
		if(!section) yyerror("Missing Section before inserting field\n");
		if(!contact) yyerror("Missing Contact before inserting field\n");
		
		memset(str,0,STRSIZE);
		memset(str2,0,STRSIZE);
		strcpy(str2,yytext+1);
		
		/* insulate the tag of the field */
		i=0;
		while(str2[i]!=':')
		{
			if(i==STRSIZE)
			{
				yyerror("Could not match a field\n");
			}
			++i;
		}
					
		str2[i]='\0';
		
		/*find the field item*/
		strcpy(str,yytext+1+i); 
		
		/*field relevation*/
		if(strcmp(str2,"name")==0)
		{
			printf("<li>Name: <b>%s</b></li>\n",str);
		}
		else if(strcmp(str2,"sur")==0)
		{
			printf("<li>Surname: <b>%s</b></li>\n",str);
		}
		else if(strcmp(str2,"add")==0)
		{
			printf("<li>Address: %s</li>\n",str);
		}
		else if(strcmp(str2,"city")==0)
		{
			printf("<li>City: %s</li>\n",str);
		}
		else if(strcmp(str2,"tel")==0)
		{
			printf("<li>Telephone: %s</li>\n",str);
		}
		else if(strcmp(str2,"cell")==0)
		{
			printf("<li>Cellphone: %s</li>\n",str);
		}
		else if(strcmp(str2,"email")==0)
		{
			printf("<li>Email: <a href=\"mailto:%s\">%s</a></li>\n",str,str);
		}
		else if(strcmp(str2,"im")==0)
		{
			printf("<li>Instant Messenging: %s</li>\n",str);
		}
		else if(strcmp(str2,"web")==0)
		{
			printf("<li>Website: <a href=\"%s\">%s</a></li>\n",str,str);
		}
		else if(strcmp(str2,"blog")==0)
		{
			printf("<li>Weblog: <a href=\"%s\">%s</a></li>\n",str,str);
		}
		else
		{
			yyerror("Tag not recognized\n");
		}
	}
			}
%%

int yywrap(void) {
	return 1;
}

void yyerror(char *err)
{
	fprintf(stderr,"\n  ERROR: %s\n",err);
	exit(1);
}

int main(int argc, char *argv[])
{
	
	owner=0;
	date=0;
	section=0;
	contact=0;
	memset(str,0,STRSIZE);
	memset(str2,0,STRSIZE);
	i=0;
	
	if(argc<2)
	{
		yyerror("Usage     $ memo2html yourcontacts.rub > rub.html\nThanks for try to use memo2html by Giovanni Santostefano\n");
	}
	
	yyin=fopen(argv[1],"r");
	
	if(!yyin)
	{
		yyerror("Could not open file\n");
	}
	
	printf("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0.1//EN\">\n<!--Generated by quanta Plus template - freely use and distribute-->\n<html>\n<head>\n<title>My Memo</title>\n<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n<meta name=\"GENERATOR\" content=\"Quanta Plus\">\n<link><style type=\"text/css\"></style>\n</head>\n<body>\n\n");
	
	yylex();
	
	if(!owner) yyerror("Missing Owner\n");
	if(!date) yyerror("Missing Date\n");
	if(section!=0) yyerror("Section not closed\n");
	if(contact!=0) yyerror("Contact not closed\n");
	printf("<br><br><br><p align=\"center\"><center><h6>Memo generated by memo2html by <a href=\"http://santostefanogiovanni.blogspot.com\">Giovanni Santostefano</a></h6></center></p><br>\n</body>\n</html>");
	
	fclose(yyin);
	return 0;
}

